searchData={"items":[{"type":"module","title":"AttrAccessor","doc":"Ruby style accessors for Elixir structs. See [README](./README.md) for more example usage.","ref":"AttrAccessor.html"},{"type":"macro","title":"AttrAccessor.attr_accessor/1","doc":"Creates `attr_reader/1` + `attr_updater/1` + `attr_writer/1` functions on struct module for `attr` symbol/symbols","ref":"AttrAccessor.html#attr_accessor/1"},{"type":"macro","title":"Example - AttrAccessor.attr_accessor/1","doc":"defmodule MyStruct do\n      defstruct [:key1, :key2, :key3]\n\n      import AttrAccessor\n      attr_accessor :key1 # single key, or multiple is ok\n      attr_accessor [:key2, :key3]\n    end\n\n    my_struct = %MyStruct{key1: \"foo\", key2: \"bar\", key3: \"baz\"}\n\n    my_struct\n      |> MyStruct.key1(\"baz\")\n      |> MyStruct.key1(&String.reverse/1)\n      |> MyStruct.key1()\n    # \"zab\"\n\n\nWriting something like\n\n    defmodule MyStruct do\n      defstruct [:some_key]\n      attr_accessor :some_key\n    end\n\nIs equivalent to\n\n    defmodule MyStruct do\n      defstruct [:some_key]\n      attr_reader :some_key\n      attr_updater :some_key\n      attr_writer :some_key\n    end\n\nWhich is also equivalent to\n\n    defmodule MyStruct do\n      defstruct [:some_key]\n\n      def some_key(st = %__MODULE__{some_key: value}) do\n        value\n      end\n\n      def some_key!(st = %__MODULE__{some_key: value}, f) do\n        %__MODULE__{ st | some_key: f.(value) }\n      end\n\n      def some_key(st = %__MODULE__{}, value) do\n        %__MODULE__{ st | some_key: value }\n      end\n    end","ref":"AttrAccessor.html#attr_accessor/1-example"},{"type":"macro","title":"AttrAccessor.attr_reader/1","doc":"Creates reader functions on struct module for `attr` symbol/symbols","ref":"AttrAccessor.html#attr_reader/1"},{"type":"macro","title":"Example - AttrAccessor.attr_reader/1","doc":"defmodule MyStruct do\n      defstruct [:key1, :key2, :key3]\n\n      import AttrAccessor\n      attr_reader :key1 # single key, or multiple is ok\n      attr_reader [:key2, :key3]\n    end\n\n    my_struct = %MyStruct{key1: \"foo\", key2: \"bar\", key3: \"baz\"}\n\n    MyStruct.key1(my_struct)\n    # \"foo\"\n\n\nWriting something like\n\n    defmodule MyStruct do\n      defstruct [:some_key]\n      attr_reader :some_key\n    end\n\nIs equivalent to \n\n    defmodule MyStruct do\n      defstruct [:some_key]\n\n      def some_key(st = %__MODULE__{some_key: value}) do\n        value\n      end\n    end","ref":"AttrAccessor.html#attr_reader/1-example"},{"type":"macro","title":"AttrAccessor.attr_updater/1","doc":"Creates a \"bang\" updater functions on struct module for `attr` symbol/symbols which accept a function to read the current value, pass it to the function, and write the result back to the struct.","ref":"AttrAccessor.html#attr_updater/1"},{"type":"macro","title":"Example - AttrAccessor.attr_updater/1","doc":"defmodule MyStruct do\n      defstruct [:key1, :key2, :key3]\n\n      import AttrAccessor\n      attr_updater :key1 # single key, or multiple is ok\n      attr_updater [:key2, :key3]\n    end\n\n    my_struct =\n      %MyStruct{key1: \"foo\"}\n      |> MyStruct.key1!(&String.upcase/1)\n\n    my_struct.key1\n    # \"FOO\"\n\nWriting something like\n\n    defmodule MyStruct do\n      defstruct [:some_key]\n      attr_updater :some_key\n    end\n\nIs equivalent to \n\n    defmodule MyStruct do\n      defstruct [:some_key]\n\n      def some_key!(st = %__MODULE__{some_key: value}, f) do\n        %__MODULE__{ st | some_key: f.(value) }\n      end\n    end","ref":"AttrAccessor.html#attr_updater/1-example"},{"type":"macro","title":"AttrAccessor.attr_writer/1","doc":"Creates writer functions on struct module for `attr` symbol/symbols.","ref":"AttrAccessor.html#attr_writer/1"},{"type":"macro","title":"Example - AttrAccessor.attr_writer/1","doc":"defmodule MyStruct do\n      defstruct [:key1, :key2, :key3]\n\n      import AttrAccessor\n      attr_writer :key1 # single key, or multiple is ok\n      attr_writer [:key2, :key3]\n    end\n\n    my_struct =\n      %MyStruct{key1: \"foo\"}\n      |> MyStruct.key1(\"bar)\n\n    my_struct.key1\n    # \"bar\"\n\nWriting something like\n\n    defmodule MyStruct do\n      defstruct [:some_key]\n      attr_writer :some_key\n    end\n\nIs equivalent to \n\n    defmodule MyStruct do\n      defstruct [:some_key]\n\n      def some_key(st = %__MODULE__{}, value) do\n        %__MODULE__{ st | some_key: value }\n      end\n    end","ref":"AttrAccessor.html#attr_writer/1-example"},{"type":"extras","title":"AttrAccessor","doc":"# AttrAccessor\n\n[Ruby style accessors](https://ruby-doc.org/docs/ruby-doc-bundle/UsersGuide/rg/accessors.html) for Elixir structs.\n\nProvides `AttrAccessor` module provides [`attr_reader`](`AttrAccessor.attr_reader/1`), [`attr_writer`](`AttrAccessor.attr_writer/1`) and [`attr_accessor`](`AttrAccessor.attr_accessor/1`) macros to define \"get\" and/or \"set\" functions for reading/write struct field values.","ref":"readme.html"},{"type":"extras","title":"Benefits - AttrAccessor","doc":"- accessors are functions, so can be used in pipe chain\n- accessors are functions, so can be passed to `Enum` etc higher order functions\n- opaque structs can more easily keep implementation details separate","ref":"readme.html#benefits"},{"type":"extras","title":"Basic Usage - AttrAccessor","doc":"Say we've got an basic \"todo\" item struct define as:\n\n```elixir\ndefmodule Todo do\n  defstruct [:id, :title, :description, :created_at, :done_at]\n\n  def done?(todo) do\n    todo.done_at != nil\n  end\nend\n```\n\nWe can use `AttrAccessor` to create get/set functions.\n\n```elixir\ndefmodule Todo do\n  defstruct [:id, :title, :description, :created_at, :done_at]\n\n  import AttrAccessor\n  attr_accessor [:title, :description]\n  attr_reader [:id, :created_at]\n  attr_writer :done_at\n\n  def done?(todo) do\n    todo.done_at != nil\n  end\nend\n  ```\n\nThen we can use those accessor functions to read and write data on the struct value.\n\n```elixir\nmy_todo = %Todo{id: 123, created_at: DateTime.utc_now()}\n\nTodo.id(my_todo)\n# 123\n\nmy_todo = Todo.title(\"my todo item\")\nmy_todo = Todo.description(\"do some stuff\")\n\nTodo.title(my_todo)\n# \"my todo item\"\n\nTodo.description(my_todo)\n# \"do some stuff\"\n\nTodo.done?(my_todo)\n# false\n\nmy_todo = Todo.done_at(my_todo, DateTime.utc_now())\nTodo.done?(my_todo)\n# true\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Mapping update functions - AttrAccessor","doc":"`AttrAccessor` also creates a \"bang\" method for each [`attr_writer`](`AttrAccessor.attr_writer/1`) defined on a struct to allow the current value to be passed to a function, and the resulting value set.\n\n```elixir\nmy_todo = %Todo{} |> Todo.title(\"My todo\")\n# %Todo{title: \"My todo\"}\n\nmy_todo = my_todo |> Todo.title!(&String.upcase/1)\n# %Todo{title: \"MY TODO\"}\n```","ref":"readme.html#mapping-update-functions"},{"type":"extras","title":"Pipe friendly - AttrAccessor","doc":"On it's own `AttrAccessor` may not provide much over built in syntax for accessing struct fields, but because they are regular Elixir functions they can be used with pipe chains to incrementally build a struct.\n\n```elixir\n%Todo{id: 123, created_at: DateTime.utc_now()}\n|> Todo.title(\"my todo item\")\n|> Todo.description(\"do some stuff\")\n|> Todo.done_at(DateTime.utc_now())\n```","ref":"readme.html#pipe-friendly"},{"type":"extras","title":"Using as higher order functions - AttrAccessor","doc":"Because accessors are functions, they can also be used with `Enum` etc functions\n\n```elixir\ntodos = [\n  %Todo{title: \"do thing 1\"},\n  %Todo{title: \"do thing 2\"},\n]\n\ntodo_titles = todos |> Enum.map(&Todo.title/1)\n# [\"do thing 1\", \"do thing 2\"]\n\nnow = DateTime.utc_now()\ncompleted_todos = todos |> Enum.map(&Todo.done_at(&1, now))\n\ncompleted_todos |> Enum.map(&Todo.done?/1)\n# [true, true]\n```","ref":"readme.html#using-as-higher-order-functions"},{"type":"extras","title":"Installation - AttrAccessor","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `attr_accessor` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:attr_accessor, \"~> 0.1.0\"}\n  ]\nend\n```\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#installation"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}